################################################################################# 
## Server Health Check
## Roberto Bonfa'
## 19.10.2018 
## Versione : 1.6 
## Script creato per la verifica dell'utilizzo di CPU - RAM - DISCHI (C e D)
## Lo script richiede come requisito la creazione del file  ServerList.txt
## 	+ il file ServerList.txt deve trovarsi nella stessa directory dello script
## 	+ il file ServerList.txt deve contenere i nomi delle macchine da verificare
## 	+ i nomi delle macchine nel file ServerList.txt devono essere uno per riga
################################################################################ 

# Funzione per raccogliere dati dal server
function Get-ServerData($computername) {
    try {
        $procLoad = (Get-WmiObject -ComputerName $computername -Class win32_processor |
                    Measure-Object -Property LoadPercentage -Average).Average
        $os = Get-WmiObject -Class win32_operatingsystem -ComputerName $computername
        $memUsage = ("{0:N2}" -f ((($os.TotalVisibleMemorySize - $os.FreePhysicalMemory) * 100) / $os.TotalVisibleMemorySize))

        $disks = Get-WmiObject -Class win32_Volume -ComputerName $computername | Where-Object { $_.DriveLetter -match "C:|D:" }
        $diskInfo = @{}
        foreach ($disk in $disks) {
            $diskInfo[$disk.DriveLetter] = ("{0:N2}" -f (($disk.FreeSpace / $disk.Capacity) * 100))
        }

        return [PSCustomObject]@{
            ServerName = $computername
            CPULoad = "${procLoad}%"
            MemLoad = "${memUsage}%"
            CDrive = "$($diskInfo['C:'])%"
            DDrive = "$($diskInfo['D:'])%"
        }
    } catch {
        Write-Host "Errore durante l'accesso a $computername: $_"
    }
}

# Creazione del report HTML
function Create-HTMLReport($data) {
    $html = "<HTML><TITLE> Server Health Report </TITLE><BODY><H2> Server Health Report </H2>"
    $html += "<Table border=1><TR><TD>Nome Server</TD><TD>% Utilizzo CPU</TD><TD>% Utilizzo RAM</TD><TD>% spazio libero disco C</TD><TD>% spazio libero disco D</TD></TR>"

    foreach ($entry in $data) {
        $rowColor = if ($entry.CPULoad -ge "80%" -or $entry.MemLoad -ge "80%") { "bgcolor='yellow'" } else { "" }
        $html += "<TR $rowColor><TD>$($entry.ServerName)</TD><TD>$($entry.CPULoad)</TD><TD>$($entry.MemLoad)</TD><TD>$($entry.CDrive)</TD><TD>$($entry.DDrive)</TD></TR>"
    }

    $html += "</Table></BODY></HTML>"
    return $html
}

# Script principale
$ServerList = Get-Content ".\ServerList.txt" -ErrorAction SilentlyContinue
$Results = foreach ($server in $ServerList) {
    Get-ServerData -computername $server
}

$Report = Create-HTMLReport -data $Results
$Report | Out-File ".\$(get-date -f dd.MM.yyyy_ore_HH) - health_report.htm"

#Invoke-Expression ".\health_report.htm"

#######Inviamo la mail con il check allegato
#$smtpServer = "yoursmtpserver.com"
#$smtpFrom = "fromemailaddress@test.com"
#$smtpTo = "receipentaddress@test.com"
#$messageSubject = "Servers Health report"
#$message = New-Object System.Net.Mail.MailMessage $smtpfrom, $smtpto
#$message.Subject = $messageSubject
#$message.IsBodyHTML = $true
#$message.Body = "<head><pre>$style</pre></head>"
#$message.Body += Get-Content C:\scripts\test.htm
#$smtp = New-Object Net.Mail.SmtpClient($smtpServer)
#$smtp.Send($message)

#Eseguiamo anche il secondo script, automaticamente, al termine di questo
& ".\performance_check.ps1"