################################################################################# 
## Performance Check
## Roberto Bonf�
## 24.10.2018
## Versione : 1.4 
## Script creato per la verifica dell'utilizzo di CPU - RAM dai processi
################################################################################

# Funzioni per raccogliere dati
function Get-ProcessMemoryUsage($computername) {
    try {
        Get-WmiObject -ComputerName $computername Win32_Process |
        Sort WorkingSetSize -Descending |
        Select Name, CommandLine, @{n="Private Memory (MB)"; e={[math]::round(($_.WorkingSetSize / 1mb), 2)}} |
        Select -First 10
    } catch {
        Write-Host "Errore nel raccogliere dati dei processi da $computername: $_"
    }
}

function Get-ServerMemoryInfo($computername) {
    try {
        $fields = @(
            @{Name = "Memory Percentage in Use (%)"; Expression = { "{0:N2}" -f ((($_.TotalVisibleMemorySize - $_.FreePhysicalMemory) * 100) / $_.TotalVisibleMemorySize) }},
            @{label = "Available Physical Memory (MB)"; Expression = {[math]::round(($_.FreePhysicalMemory / 1kb), 2)}},
            @{label = "Total Physical Memory (MB)"; Expression = {[math]::round(($_.TotalVisibleMemorySize / 1kb), 2)}},
            @{label = "Available Virtual Memory (MB)"; Expression = {[math]::round(($_.FreeVirtualMemory / 1kb), 2)}},
            @{label = "Total Virtual Memory (MB)"; Expression = {[math]::round(($_.TotalVirtualMemorySize / 1kb), 2)}}
        )
        Get-WmiObject -Class win32_OperatingSystem -ComputerName $computername | Format-List $fields
    } catch {
        Write-Host "Errore nel raccogliere informazioni di memoria da $computername: $_"
    }
}

# Script principale
$ServerList = Get-Content ".\ServerList.txt" -ErrorAction SilentlyContinue
$performance_check = ForEach($computername in $ServerList) {
    "===========Memory - Top 10 Utilization List - $computername==========="
    Get-ProcessMemoryUsage -computername $computername | Out-String

    "===========Server Memory Information - $computername==========="
    Get-ServerMemoryInfo -computername $computername
}

# Salvataggio del report
$performance_check | Out-File -FilePath "$(get-date -f dd.MM.yyyy_ore_HH) - Performance_Check.txt"